import * as grammar from "../../../grammar/interface";
import * as tokenLevel from "../../../tokenLevel/interface";
export declare type WrappedStringType = ["apostrophe", {}] | ["quote", {}] | [
    "multiline",
    {
        previousLines: string[];
    }
];
/**
 * A PreToken is a low level token
 */
export declare type PreToken = {
    type: [
        "header start",
        {
            range: tokenLevel.Range;
        }
    ] | [
        "block comment begin",
        {
            range: tokenLevel.Range;
        }
    ] | [
        "block comment end",
        {
            range: tokenLevel.Range;
        }
    ] | [
        "line comment begin",
        {
            range: tokenLevel.Range;
        }
    ] | [
        "line comment end",
        {
            location: tokenLevel.Location;
        }
    ] | [
        "newline",
        {
            range: tokenLevel.Range;
        }
    ] | [
        "structural",
        {
            type: grammar.StructuralTokenType;
            range: tokenLevel.Range;
        }
    ] | [
        "wrapped string begin",
        {
            range: tokenLevel.Range;
            type: WrappedStringType;
        }
    ] | [
        "wrapped string end",
        {
            range: tokenLevel.Range;
            wrapper: string | null;
        }
    ] | [
        "snippet",
        {
            chunk: string;
            begin: number;
            end: number;
        }
    ] | [
        "non wrapped string begin",
        {
            location: tokenLevel.Location;
        }
    ] | [
        "non wrapped string end",
        {
            location: tokenLevel.Location;
        }
    ] | [
        "whitespace begin",
        {
            location: tokenLevel.Location;
        }
    ] | [
        "whitespace end",
        {
            location: tokenLevel.Location;
        }
    ];
};
