"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createBuilder = void 0;
function createBuilder(ds, onEnd) {
    function buildValue(value) {
        return {
            onDictionary: () => {
                const dict = {};
                value.type = ["dictionary", dict];
                return {
                    onClose: () => { },
                    onEntry: ($) => {
                        const entry = { type: null };
                        dict[$.token.token.value] = entry;
                        return buildValue(entry);
                    },
                };
            },
            onList: () => {
                const list = [];
                value.type = ["list", list];
                return {
                    onClose: () => { },
                    onElement: () => {
                        const element = { type: null };
                        list.push(element);
                        return buildValue(element);
                    },
                };
            },
            onTaggedUnion: ($) => {
                const taggedUnion = {
                    option: null,
                    value: { type: null },
                };
                value.type = ["tagged union", taggedUnion];
                return {
                    onUnexpectedOption: () => {
                        taggedUnion.option = $.definition["default option"].name;
                        return buildValue(taggedUnion.value);
                    },
                    onOption: ($$) => {
                        taggedUnion.option = $$.name;
                        return buildValue(taggedUnion.value);
                    },
                    onEnd: () => { },
                };
            },
            onSimpleString: ($) => {
                value.type = ["simple string", $.value];
            },
            onMultilineString: ($) => {
                value.type = ["multiline string", $.token === null ? [] : $.token.token.lines];
            },
            onTypeReference: () => {
                return buildValue(value);
            },
            onGroup: () => {
                const group = {};
                value.type = ["group", group];
                return {
                    onUnexpectedProperty: () => { },
                    onProperty: ($) => {
                        const property = { type: null };
                        group[$.key] = property;
                        return buildValue(property);
                    },
                    onClose: () => { },
                };
            },
        };
    }
    return {
        root: buildValue(ds.root),
        onEnd: () => {
            onEnd();
        },
    };
}
exports.createBuilder = createBuilder;
