"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCodeCompletionFinder = void 0;
const createCodeCompletionsGenerator_1 = require("./createCodeCompletionsGenerator");
const isPositionBeforeLocation_1 = require("./isPositionBeforeLocation");
function onPositionInContextOfRange(positionLine, positionCharacter, range, onBefore, onIn, onAfter, getEndLocationFromRange) {
    if ((0, isPositionBeforeLocation_1.isPositionBeforeLocation)({
        positionLine: positionLine,
        positionCharacter: positionCharacter,
        location: range.start,
    })) {
        onBefore();
    }
    else if ((0, isPositionBeforeLocation_1.isPositionBeforeLocation)({
        positionLine: positionLine,
        positionCharacter: positionCharacter,
        location: getEndLocationFromRange(range),
    })) {
        onIn();
    }
    else {
        onAfter();
    }
}
function createCodeCompletionFinder(completionPositionLine, completionPositionCharacter, callback, getEndLocationFromRange) {
    let positionAlreadyFound = false;
    let previousAfter = null;
    //console.log("FINDING COMPLETIONS", line, character)
    function generate(gs) {
        if (gs !== null) {
            const codeCompletions = gs();
            //console.log(codeCompletions)
            codeCompletions.forEach((codeCompletion) => {
                //console.log("codeCompletion", codeCompletion)
                callback(codeCompletion);
            });
        }
    }
    return (0, createCodeCompletionsGenerator_1.createCodeCompletionsGenerator)((annotation, intra, after) => {
        if (!positionAlreadyFound) {
            onPositionInContextOfRange(completionPositionLine, completionPositionCharacter, annotation.range, () => {
                generate(previousAfter);
                positionAlreadyFound = true;
            }, () => {
                generate(intra);
                positionAlreadyFound = true;
            }, () => {
                previousAfter = after;
            }, getEndLocationFromRange);
        }
    }, () => {
        if (!positionAlreadyFound) {
            generate(previousAfter);
        }
    });
}
exports.createCodeCompletionFinder = createCodeCompletionFinder;
