"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCodeCompletionsGenerator = void 0;
const stringSerialization_1 = require("../../../astn/esc/implementation/stringSerialization");
const pr = __importStar(require("pareto-runtime"));
function createCodeCompletionForValue(value, sequence, onTaggedUnion, onGroup) {
    switch (value.type[0]) {
        case "dictionary": {
            sequence.snippet(` { }`);
            break;
        }
        case "list": {
            sequence.snippet(` [ ]`);
            break;
        }
        case "type reference": {
            const $ = value.type[1];
            createCodeCompletionForValue($.type.get().value, sequence, onTaggedUnion, onGroup);
            break;
        }
        case "tagged union": {
            const $ = value.type[1];
            onTaggedUnion($);
            break;
        }
        case "simple string": {
            const $ = value.type[1];
            if ($.quoted) {
                sequence.snippet(` "${$["default value"]}"`);
            }
            else {
                sequence.snippet(` ${$["default value"]}`);
            }
            break;
        }
        case "multiline string": {
            sequence.snippet(` \`\``);
            break;
        }
        case "group": {
            const $ = value.type[1];
            onGroup($);
            break;
        }
        default:
            pr.au(value.type[0]);
    }
}
function createCodeCompletionForShorthandValue(definition, sequence) {
    createCodeCompletionForValue(definition, sequence, ($) => {
        const step = sequence.addTaggedUnionStep();
        $.options.forEach((option, optionName) => {
            const seq = step.addOption();
            seq.snippet(` '${optionName}'`);
            createCodeCompletionForShorthandValue(option.value, seq);
        });
    }, ($) => {
        createCodeCompletionForShorthandGroup($, sequence);
    });
}
function createCodeCompletionForShorthandGroup(group, sequence) {
    group.properties.forEach((prop, _propKey) => {
        createCodeCompletionForShorthandValue(prop.value, sequence);
    });
}
function createCodeCompletionsForTaggedUnion($, sequence) {
    sequence.snippet(` '${$["default option"].name}'`);
    createCodeCompletionsForValue($["default option"].get().value, sequence);
}
function createCodeCompletionForVerboseValue(prop, sequence) {
    createCodeCompletionForValue(prop, sequence, ($) => {
        sequence.snippet(` | '${$["default option"].name}'`);
        createCodeCompletionForVerboseValue($["default option"].get().value, sequence);
    }, ($) => {
        sequence.snippet(` (`);
        createCodeCompletionForVerboseProperties($, sequence);
        sequence.snippet(`)`);
    });
}
function createCodeCompletionForVerboseProperties(group, sequence) {
    let dirty = false;
    sequence.indent(($) => {
        group.properties.forEach((prop, propKey) => {
            dirty = true;
            const line = $.addLine();
            line.snippet(`'${propKey}':`);
            createCodeCompletionForVerboseValue(prop.value, line);
        });
    });
    if (!dirty) {
        sequence.snippet(' ');
    }
}
function createAlternativesRoot() {
    const rootSequence = [];
    function createBlock(imp) {
        return {
            addLine: () => {
                const seq = [];
                imp.lines.push(seq);
                return createSequence(seq);
            },
        };
    }
    function createSequence(imp) {
        return {
            indent: (callback) => {
                const block = {
                    lines: [],
                };
                imp.push(["block", {
                        block: block,
                    }]);
                callback(createBlock(block));
            },
            snippet: (str) => {
                imp.push(["snippet", { value: str }]);
            },
            addTaggedUnionStep: () => {
                function createStep(sequence) {
                    const alts = [];
                    sequence.push(["tagged union", { alts: alts }]);
                    return {
                        addOption: () => {
                            const subSeq = [];
                            alts.push(subSeq);
                            return createSequence(subSeq);
                        },
                    };
                }
                return createStep(imp);
            },
        };
    }
    return {
        root: createSequence(rootSequence),
        serialize: () => {
            let indentationLevel = 0;
            function createIndentation() {
                let str = "";
                for (let i = 0; i !== indentationLevel; i += 1) {
                    str += "    ";
                }
                return str;
            }
            function ser(seed, s, add) {
                let out = seed;
                for (let i = 0; i !== s.length; i += 1) {
                    const step = pr.getElement(s, i);
                    switch (step[0]) {
                        case "block":
                            pr.cc(step[1], (step2) => {
                                indentationLevel += 1;
                                step2.block.lines.forEach((l) => {
                                    const temp = [];
                                    ser(out.map((str) => `${str}\n${createIndentation()}`), l, (str) => temp.push(str));
                                    out = temp;
                                });
                                indentationLevel -= 1;
                                if (step2.block.lines.length !== 0) {
                                    out = out.map((str) => `${str}\n${createIndentation()}`);
                                }
                            });
                            break;
                        case "snippet":
                            pr.cc(step[1], (step2) => {
                                out = out.map((str) => {
                                    return str + step2.value;
                                });
                            });
                            break;
                        case "tagged union":
                            pr.cc(step[1], (step2) => {
                                const temp = [];
                                for (let j = 0; j !== step2.alts.length; j += 1) {
                                    const alt = pr.getElement(step2.alts, j);
                                    ser(out, alt, (str) => temp.push(str));
                                }
                                out = temp;
                            });
                            break;
                        default:
                            pr.au(step[0]);
                    }
                }
                out.forEach((str) => {
                    add(str);
                });
            }
            const res = [];
            ser([""], rootSequence, (str) => res.push(str));
            return res;
        },
    };
}
function createCodeCompletionsForValue(definition, line) {
    createCodeCompletionForValue(definition, line, ($) => {
        line.snippet(` |`);
        createCodeCompletionsForTaggedUnion($, line);
    }, ($) => {
        const tus = line.addTaggedUnionStep();
        const verbose = tus.addOption();
        verbose.snippet(` (`);
        createCodeCompletionForVerboseProperties($, verbose);
        verbose.snippet(`)`);
        const shorthand = tus.addOption();
        shorthand.snippet(` <`);
        createCodeCompletionForShorthandGroup($, shorthand);
        shorthand.snippet(` >`);
    });
}
function createCodeCompletionsGenerator(onToken, onEnd) {
    function createValueHandler() {
        function ifToken(token, inToken, afterToken) {
            if (token === null) {
                return;
            }
            onToken(token.annotation, inToken, afterToken);
        }
        function addDummyOnToken(token) {
            ifToken(token, null, null);
        }
        function doGroup(annotation, alternatives) {
            if (annotation !== null) {
                onToken(annotation, null, () => {
                    return alternatives;
                });
            }
            return {
                onProperty: ($) => {
                    ifToken($.token, null, () => {
                        const propAlts = createAlternativesRoot();
                        createCodeCompletionsForValue($.definition, propAlts.root);
                        return propAlts.serialize();
                    });
                    return createValueHandler();
                },
                onUnexpectedProperty: ($) => {
                    onToken($.token.annotation, () => {
                        return $.expectedProperties;
                    }, null);
                },
                onClose: ($$) => {
                    ifToken($$.token, () => {
                        return alternatives;
                    }, null);
                },
            };
        }
        return {
            onDictionary: ($) => {
                addDummyOnToken($.token);
                return {
                    onClose: ($) => {
                        addDummyOnToken($.token);
                    },
                    onEntry: ($$) => {
                        ifToken($$.token, null, () => {
                            const entryAlts = createAlternativesRoot();
                            createCodeCompletionsForValue($.definition.value, entryAlts.root);
                            return entryAlts.serialize();
                        });
                        return createValueHandler();
                    },
                };
            },
            onList: ($) => {
                addDummyOnToken($.token);
                return {
                    onClose: ($) => {
                        addDummyOnToken($.token);
                    },
                    onElement: () => {
                        return createValueHandler();
                    },
                };
            },
            onTaggedUnion: ($) => {
                ifToken($.token, null, () => {
                    const alternatives = createAlternativesRoot();
                    createCodeCompletionsForTaggedUnion($.definition, alternatives.root);
                    return alternatives.serialize();
                });
                return {
                    onUnexpectedOption: ($$) => {
                        onToken($$.token.annotation, () => {
                            return $$.expectedOptions;
                        }, null);
                        return createValueHandler();
                    },
                    onOption: ($$) => {
                        addDummyOnToken($$.token);
                        return createValueHandler();
                    },
                    onEnd: () => { },
                };
            },
            onSimpleString: ($) => {
                ifToken($.token, () => {
                    return [
                        $.definition.quoted
                            ? (0, stringSerialization_1.createSerializedQuotedString)($.definition["default value"])
                            : (0, stringSerialization_1.createSerializedNonWrappedString)($.definition["default value"]),
                    ];
                }, 
                // () => {
                //     return $.getSuggestions().map(sugg => {
                //         return $.definition.quoted ? `"${sugg}"` : sugg
                //     })
                // },
                null);
            },
            onMultilineString: ($) => {
                addDummyOnToken($.token);
            },
            onTypeReference: () => {
                return createValueHandler();
            },
            onGroup: ($) => {
                const definition = $.definition;
                switch ($.type[0]) {
                    case "mixin":
                        return doGroup(null, []);
                    case "omitted":
                        return doGroup(null, []);
                    case "shorthand":
                        return pr.cc($.type[1], ($) => {
                            const alternatives = createAlternativesRoot();
                            createCodeCompletionForShorthandGroup(definition, alternatives.root);
                            return doGroup($.annotation, alternatives.serialize());
                        });
                    case "verbose":
                        return pr.cc($.type[1], ($) => {
                            const alternatives = createAlternativesRoot();
                            createCodeCompletionForVerboseProperties(definition, alternatives.root);
                            return doGroup($.annotation, alternatives.serialize());
                        });
                    default:
                        return pr.au($.type[0]);
                }
            },
        };
    }
    return {
        root: createValueHandler(),
        onEnd: onEnd,
    };
}
exports.createCodeCompletionsGenerator = createCodeCompletionsGenerator;
