"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.marshall = exports.marshallDataset = void 0;
const pr = __importStar(require("pareto-runtime"));
const serializeSchema_generated_1 = require("../../../schema/esc/implementation/serializeSchema.generated");
const stringSerialization_1 = require("../../../astn/esc/implementation/stringSerialization");
const pareto_runtime_1 = require("pareto-runtime");
function onValueIsNonDefault(value, definition, callback) {
    switch (definition.type[0]) {
        case "type reference": {
            const $ = definition.type[1];
            onValueIsNonDefault(value, $.type.get().value, callback);
            break;
        }
        case "dictionary": {
            value.toDictionary((dict) => {
                if (!dict.entries.isEmpty()) {
                    callback();
                }
            });
            break;
        }
        case "list": {
            value.toList((list) => {
                if (!list.elements.isEmpty()) {
                    callback();
                }
            });
            break;
        }
        case "tagged union": {
            const $ = definition.type[1];
            value.toTaggedUnion((tu) => {
                if (tu.option !== $["default option"].name) {
                    callback();
                }
                else {
                    onValueIsNonDefault(tu.value, $.options.getLookup().getUnsafe(tu.option).value, callback);
                }
            });
            break;
        }
        case "simple string": {
            const $ = definition.type[1];
            value.toSimpleString((str) => {
                if (str !== $["default value"]) {
                    callback();
                }
            });
            break;
        }
        case "multiline string": {
            value.toMultilineString((lines) => {
                if (lines.length > 1) {
                    callback();
                }
                if (lines.length === 1 && (0, pareto_runtime_1.getElement)(lines, 0) !== "") {
                    callback();
                }
            });
            break;
        }
        case "group": {
            const $ = definition.type[1];
            value.toGroup((group) => {
                let foundNonDefault = false;
                $.properties.forEach((p, key) => {
                    group.onProperty(key, (value) => {
                        onValueIsNonDefault(value, p.value, () => {
                            foundNonDefault = true;
                        });
                    });
                });
                if (foundNonDefault) {
                    callback();
                }
            });
            break;
        }
        default:
            return pr.au(definition.type[0]);
    }
}
function marshallDataset(dataset, definition, out, style) {
    marshallValue(dataset.root, definition.value, out, style, false);
}
exports.marshallDataset = marshallDataset;
function marshallValue(value, definition, out, style, inMixinMode) {
    switch (definition.type[0]) {
        case "dictionary": {
            const $ = definition.type[1];
            value.toDictionary((dict) => {
                out.sendBlock({
                    open: ["structural", {
                            "type": ["open dictionary", {}],
                        }],
                    close: ["structural", {
                            "type": ["close dictionary", {}],
                        }],
                }, (out) => {
                    dict.entries.forEach((entry, key) => {
                        out.sendEvent(["simple string", {
                                value: key,
                                wrapping: ["quote", {}],
                            }]);
                        marshallValue(entry, $.value, out, style, false);
                    });
                });
            });
            break;
        }
        case "list": {
            const $$ = definition.type[1];
            value.toList((list) => {
                out.sendBlock({
                    open: ["structural", {
                            "type": ["open shorthand group", {}],
                        }],
                    close: ["structural", {
                            "type": ["close shorthand group", {}],
                        }],
                }, (out) => {
                    list.elements.forEach((e) => {
                        marshallValue(e, $$.value, out, style, false);
                    });
                });
            });
            break;
        }
        case "type reference": {
            const $ = definition.type[1];
            marshallValue(value, $.type.get().value, out, style, inMixinMode);
            break;
        }
        case "tagged union": {
            const $ = definition.type[1];
            value.toTaggedUnion((taggedUnion) => {
                if (!inMixinMode) {
                    out.sendEvent(["structural", {
                            "type": ["tagged union start", {}],
                        }]);
                }
                if (taggedUnion.option !== null) {
                    out.sendEvent(["simple string", {
                            value: taggedUnion.option,
                            wrapping: ["apostrophe", {}],
                        }]);
                    marshallValue(taggedUnion.value, $.options.getLookup().getUnsafe(taggedUnion.option).value, out, style, inMixinMode);
                }
            });
            break;
        }
        case "simple string": {
            const $ = definition.type[1];
            value.toSimpleString((str) => {
                out.sendEvent(["simple string", {
                        value: str,
                        wrapping: $.quoted
                            ? ["quote", {}]
                            : ["none", {}],
                    }]);
            });
            break;
        }
        case "multiline string": {
            value.toMultilineString((lines) => {
                out.sendEvent(["multiline string", {
                        lines: lines,
                    }]);
            });
            break;
        }
        case "group": {
            const $ = definition.type[1];
            value.toGroup((group) => {
                if (inMixinMode) {
                    $.properties.forEach((propDef, key) => {
                        group.onProperty(key, (prop) => {
                            marshallValue(prop, propDef.value, out, style, true);
                        });
                    });
                }
                else {
                    switch (style[0]) {
                        case "expanded": {
                            const expandedStyle = style[1];
                            out.sendBlock({
                                open: ["structural", {
                                        "type": ["open verbose group", {}],
                                    }],
                                close: ["structural", {
                                        "type": ["close verbose group", {}],
                                    }],
                            }, (out) => {
                                $.properties.forEach((propDef, key) => {
                                    group.onProperty(key, (prop) => {
                                        function serializeProperty() {
                                            out.sendEvent(["simple string", {
                                                    value: key,
                                                    wrapping: ["apostrophe", {}],
                                                }]);
                                            marshallValue(prop, propDef.value, out, style, false);
                                        }
                                        if (expandedStyle.omitPropertiesWithDefaultValues) {
                                            onValueIsNonDefault(prop, propDef.value, () => {
                                                serializeProperty();
                                            });
                                        }
                                        else {
                                            serializeProperty();
                                        }
                                    });
                                });
                            });
                            break;
                        }
                        case "compact": {
                            out.sendBlock({
                                open: ["structural", {
                                        "type": ["open shorthand group", {}],
                                    }],
                                close: ["structural", {
                                        "type": ["close shorthand group", {}],
                                    }],
                            }, (out) => {
                                $.properties.forEach((propDef, key) => {
                                    group.onProperty(key, (prop) => {
                                        marshallValue(prop, propDef.value, out, style, true);
                                    });
                                });
                            });
                            break;
                        }
                        default:
                            pr.au(style[0]);
                    }
                }
            });
            break;
        }
        default:
            pr.au(definition.type[0]);
    }
}
function marshall(dataset, schema, internalSchemaSpecification, style, writer, annotate, createTreeParser, createASTNNormalizer) {
    const newline = "\r\n";
    const indentationString = "    ";
    const writer2 = {
        token: (instruction) => {
            writer(instruction.stringBefore);
            writer(instruction.token);
            writer(instruction.stringAfter);
        },
        nonToken: (instruction) => {
            writer(instruction.string);
        },
    };
    function handleEvent(event, annotation, parser) {
        parser.onToken({
            annotation: annotation,
            token: event,
        });
    }
    switch (internalSchemaSpecification[0]) {
        case "embedded": {
            writer(`! ! "astn/schema@0.1" `);
            const embeddedSchemaParser = createTreeParser({
                handler: annotate(createASTNNormalizer({
                    indentationString: indentationString,
                    newline: newline,
                }, {
                    writer: writer2,
                })),
                onError: ($) => {
                    throw new Error(`unexpected error in schema`);
                },
            });
            (0, serializeSchema_generated_1.serializeSchema)(schema, (event) => {
                handleEvent(event, null, embeddedSchemaParser);
            });
            break;
        }
        case "none": {
            break;
        }
        case "reference": {
            const $ = internalSchemaSpecification[1];
            writer(`! ${(0, stringSerialization_1.createSerializedQuotedString)($.name)}${newline}`);
            break;
        }
        default:
            pr.au(internalSchemaSpecification[0]);
    }
    const bodyParser = createTreeParser({
        handler: annotate(createASTNNormalizer({
            indentationString: indentationString,
            newline: newline,
        }, {
            writer: writer2,
        })),
        onError: () => {
            throw new Error(`unexpected error in schema`);
        },
    });
    function createOut() {
        function he(event) {
            handleEvent(event, null, bodyParser);
        }
        return {
            sendBlock: (eventpair, callback) => {
                he(eventpair.open);
                callback(createOut());
                he(eventpair.close);
            },
            sendEvent: (event) => {
                he(event);
            },
        };
    }
    marshallDataset(dataset, schema["root type"].get(), createOut(), style);
}
exports.marshall = marshall;
